import json

from conda import __version__  # NOQA
from conda.gateways import logging  # NOQA
from conda.base.context import context
from conda.auxlib.decorators import clear_memoized_methods
from conda.models.channel import prioritize_channels, Channel
from conda.models.version import VersionOrder  # NOQA
from conda.models.match_spec import MatchSpec, ExactStrMatch  # NOQA
from conda.gateways.connection.download import download  # NOQA
from conda.gateways.connection.session import CondaSession

try:
    from conda.core.subdir_data import fetch_repodata_remote_request, SubdirData
except Exception:
    fetch_repodata_remote_request = None
    from conda.core.subdir_data import SubdirData
from conda.exceptions import UnavailableInvalidChannel


_known_subdirs = None


def known_subdirs():
    global _known_subdirs
    if _known_subdirs is None:
        _known_subdirs = sorted(set(context.known_subdirs) - {"noarch"}) + ["noarch"]
    return _known_subdirs


def channel_alias():
    return context.channel_alias.base_url.rstrip("/") + "/"


def default_channels():
    return [x.base_url for x in context.default_channels]


def empty_repodata(subdir):
    return {
        "info": {"subdir": subdir},
        "packages": {},
        "packages.conda": {},
        "removed": [],
        "repodata_version": 1,
    }


_repo_tokens = None


def get_repo_tokens():
    global _repo_tokens
    if _repo_tokens is None:
        from conda.gateways import anaconda_client as ac

        _repo_tokens = {}
        for url, token in ac.read_binstar_tokens().items():
            url = "/".join(url.split("/", 3)[:3]) + "/"
            _repo_tokens[url] = token
    return _repo_tokens


def set_repo_token(url, token):
    global _repo_tokens
    if _repo_tokens is None:
        get_repo_tokens()
    url = "/".join(url.split("/", 3)[:3]) + "/"
    if token:
        _repo_tokens[url] = token
    elif url in _repo_tokens:
        del _repo_tokens[url]


def find_token(url):
    url = url.rstrip("/") + "/"
    for t_url, token in get_repo_tokens().items():
        if url.startswith(t_url):
            return token


def apply_repo_token(url):
    token = find_token(url)
    if token:
        c = Channel(url)
        c.token = token
        return c.url(True)
    return url


def _cache_clear():
    # Unfortunately conda<24 does not implement CondaSession.cache_clear.
    # This implements effectively backports the equivalent functionality
    # to older versions of conda
    if hasattr(CondaSession, "cache_clear"):
        CondaSession.cache_clear()
    elif hasattr(CondaSession._thread_local, "sessions"):
        CondaSession._thread_local.sessions.clear()
    elif hasattr(CondaSession._thread_local, "session"):
        delattr(CondaSession.thread_local, "session")


def fetch_index(channel_urls, platform, channel_alias=None, ssl_verify=None):
    try:
        context.repodata_use_zst = False
    except Exception:
        pass
    if channel_alias is not None and channel_alias != context.channel_alias:
        old_alias, context._channel_alias = context._channel_alias, channel_alias
        clear_memoized_methods(context, "channel_alias")
        Channel._reset_state()
    else:
        channel_alias = None
    if ssl_verify is not None and ssl_verify != context.ssl_verify:
        old_ssl, context.ssl_verify = context.ssl_verify, ssl_verify
        _cache_clear()
    else:
        ssl_verify = None
    if not isinstance(channel_urls, (list, tuple)):
        channel_urls = (channel_urls,)
    try:
        filtered_results = []
        for url in prioritize_channels(
            channel_urls, with_credentials=False, subdirs=(platform,)
        ):
            if url not in filtered_results and url.rstrip("/").endswith("/" + platform):
                filtered_results.append(apply_repo_token(url))
        index = empty_repodata(platform)
        for url in reversed(filtered_results):
            if hasattr(SubdirData, "repo_fetch"):
                repodata, _ = SubdirData(Channel(url)).repo_fetch.fetch_latest_parsed()
            else:
                try:
                    raw_repodata_str = fetch_repodata_remote_request(url, None, None)
                except UnavailableInvalidChannel:
                    continue
                if raw_repodata_str is None:
                    continue
                repodata = json.loads(raw_repodata_str)
            if not isinstance(repodata, dict):
                continue
            for key in ("packages", "packages.conda"):
                for fn, pkg in repodata.get(key, {}).items():
                    index[key][fn] = pkg
                    pkg["subdir"] = platform
                    pkg["channel"] = url
                    pkg["fn"] = fn
        return index
    finally:
        if channel_alias is not None:
            context._channel_alias = old_alias
            clear_memoized_methods(context, "channel_alias")
        if ssl_verify is not None:
            context.ssl_verify = old_ssl
            _cache_clear()



import sys
import shutil

from os import sep, listdir, unlink, makedirs
from os.path import join, isdir, dirname, abspath, expanduser, exists

from .utils import compute_checksums


def get_repo_api(config):
    name = oname = config.dest_channel.rstrip("/")
    if config.dest_site:
        name = join(config.dest_site, name)
    if sep != "/":
        name = name.replace("/", sep)
    name = abspath(expanduser(name))
    base = name[: -len(oname) - 1]
    return (base,)


def _join(*args):
    return join(*(a for a in args if a))


def list_objects(base, *path, repodata=None):
    dname = _join(*base, *path)
    packages = {}
    for fn in listdir(dname) if exists(dname) else ():
        fpath = join(dname, fn)
        if not isdir(fpath):
            packages[fn] = compute_checksums(fpath)
    return packages


def get_object(base, *path):
    fpath = _join(*base, *path)
    if exists(fpath):
        with open(fpath, "rb") as fp:
            return fp.read()


def del_object(base, *path, metadata=None):
    fpath = _join(*base, *path)
    if exists(fpath):
        unlink(fpath)
    if len(path) > 2:
        dname = dirname(fpath)
        if isdir(dname) and not listdir(dname):
            shutil.rmtree(dname, ignore_errors=True)


def put_object(base, *path, path_or_data=None, metadata=None):
    fpath = _join(*base, *path)
    makedirs(dirname(fpath), exist_ok=True)
    if isinstance(path_or_data, bytes):
        with open(fpath, "wb") as fp:
            fp.write(path_or_data)
    else:
        shutil.copyfile(path_or_data, fpath)


def main():
    from .common import main as _main

    _main(sys.modules[__name__])



import sys

from .utils import MirrorException

# Disable tqdm if present
try:
    from tqdm import tqdm

    tqdm._old__init__ = tqdm.__init__

    def _new_init(self, *args, **kwargs):
        kwargs["disable"] = True
        self._old__init__(*args, **kwargs)

    tqdm.__init__ = _new_init
except ImportError:
    pass


SUBDIR_MAP = {
    "osx-32": ("osx", "x86"),
    "osx-64": ("osx", "x86_64"),
    "win-32": ("win", "x86"),
    "win-64": ("win", "x86_64"),
    "win-arm64": ("win", "arm64"),
    "linux-32": ("linux", "x86"),
    "linux-64": ("linux", "x86_64"),
    "linux-aarch64": ("linux", "aarch64"),
    "linux-armv6l": ("linux", "armv6l"),
    "linux-armv7l": ("linux", "armv7l"),
    "linux-ppc64le": ("linux", "ppc64le"),
    "linux-s390x": ("linux", "s390x"),
    "noarch": (None, None),
}


def parse_subdir(subdir):
    """
    Lifted from
    https://github.com/Anaconda-Platform/anaconda-server/blob/develop/anaconda_platform/utils/__init__.py#L133-L179
    """  # noqa: E501
    plat_arch = SUBDIR_MAP.get(subdir, subdir.split("-", 1))
    if len(plat_arch) == 1:
        plat_arch.append("")
    return tuple(plat_arch)


def get_repo_api(config):
    global errors
    global inspect_conda_package
    try:
        from binstar_client.utils import get_server_api, get_config
        from binstar_client.inspect_package.conda import inspect_conda_package
        from binstar_client import errors
    except ImportError:
        raise MirrorException("Please install the 'anaconda-client' package.")
    r_config = get_config()
    dest_site = config.dest_site or r_config["default_site"]
    if dest_site in ("null", "None"):
        dest_site = next(dest_site.keys())
    kwargs = {"site": dest_site}
    if config.password:
        kwargs["token"] = config.password
    api = get_server_api(**kwargs)
    if not config.verify_ssl_dst and not api.session.verify:
        config.verify_ssl_dst = False
    api.session.verify = config.verify_ssl_mode("dst")
    config.dest_channel_alias = api.session.get(api.domain).json()["conda_url"]
    if api.domain == "https://api.anaconda.org":
        config.valid_formats = ("conda", "tarbz2")
    else:
        config.valid_formats = ("tarbz2",)
    config.can_patch = False
    api.is_public = config.access_policy != "private"
    return api


def begin_channel(api, cname, dry_run):
    try:
        api.user(cname)
    except errors.NotFound:
        raise MirrorException("Channel not found: %s" % cname)


_packages = {}


def ensure_package_exists(api, account, info, package_attrs, release_attrs, public):
    name = info["name"]
    prec = _packages.get(name)
    if prec is None:
        try:
            prec = api.package(account, name)
        except errors.NotFound:
            prec = api.add_package(
                account,
                name,
                summary=package_attrs.get("summary", None),
                attrs=package_attrs,
                public=public,
                license=info.get("license", info.get("license_family", None)),
                license_url=package_attrs.get("license_url", None),
                license_family=package_attrs.get("license_family", None),
            )
        _packages[name] = prec
    version = info["version"]
    if version in prec.setdefault("versions", []):
        return
    response = api.add_release(account, name, version, None, None, release_attrs)
    prec["versions"].append(version)
    prec.setdefault("releases", []).append(response)


def del_object(api, cname, platform, fn, metadata=None):
    response = api.remove_dist(
        cname, metadata["name"], metadata["version"], "%s/%s" % (platform, fn)
    )
    if response and response["status"] != "ok":
        raise MirrorException("%s: %s" % (response["status"], response["message"]))


def put_object(api, cname, platform, fn, path_or_data=None, metadata=None):
    info = metadata.copy()
    info["platform"], info["arch"] = parse_subdir(platform)
    try:
        package_attrs, release_attrs, file_attrs = inspect_conda_package(path_or_data)
    except Exception as exc:
        message = getattr(exc, "message", exc)
        raise MirrorException("inspect_conda_package: %s" % message)
    ensure_package_exists(api, cname, info, package_attrs, release_attrs, api.is_public)
    try:
        with open(path_or_data, "rb") as fp:
            api.upload(
                cname,
                info["name"],
                info["version"],
                "%s/%s" % (platform, fn),
                fp,
                "conda",
                description=package_attrs.get("description", None),
                attrs=info,
                dependencies=[],
            )
    except Exception as exc:
        message = getattr(exc, "message", exc)
        raise MirrorException("api.upload: %s" % message)


def main():
    from .common import main as _main

    _main(sys.modules[__name__])



import os
import re
import sys
import urllib3
import argparse

from os.path import abspath, expanduser

try:
    import ruamel.yaml as ruamel_yaml
except ImportError:
    import ruamel_yaml

from . import __version__
from .conda_interface import (
    known_subdirs,
    MatchSpec,
    __version__ as conda_version,
    default_channels as conda_default_channels,
    channel_alias as conda_channel_alias,
    context,
)
from .filter import standard_license_families


urllib3.disable_warnings()


all_access_policies = ["private", "authenticated", "public", "default"]
all_format_policies = [
    "prefer-conda",
    "prefer-tarbz2",
    "only-conda",
    "only-tarbz2",
    "transmute-conda",
    "transmute-tarbz2",
    "keep-both",
    "default",
]


class ConfigError(RuntimeError):
    pass


def _load_config(config_path):
    if not config_path:
        return {}
    try:
        return ruamel_yaml.safe_load(open(config_path)) or {}
    except Exception as e:
        msg = 'Unable to load config file "{}": {}'
        raise ConfigError(msg.format(config_path, e))


def _get_str_list(value, unique=True):
    if value is None:
        result = []
    elif isinstance(value, (list, tuple)):
        result = [str(v) for v in value]
    elif isinstance(value, dict):
        assert False
    else:
        result = [str(value)]
    if unique:
        new_result = []
        for x in result:
            if x not in new_result:
                new_result.append(x)
        return new_result
    return result


class Config(object):
    def __init__(self, dest_channel=None, path=None):
        self.path = path
        rc = _load_config(self.path)
        self.strict_platforms = rc.get("strict_platforms", False)
        self.channel_alias = rc.get("channel_alias", None)
        self.dest_site = rc.get("dest_site")
        self.dest_channel = dest_channel or rc.get("dest_channel", None)
        self.dest_channel_alias = rc.get("dest_channel_alias", None)
        self.max_retries = int(rc.get("max_retries", 0))
        self.access_policy = rc.get("access_policy", None)
        self.format_policy = rc.get("format_policy", "default")
        self.max_attempts = rc.get("max_attempts", 5)
        self.max_failures = rc.get("max_failures", 100)
        self.verify_ssl = rc.get("verify_ssl", True)
        self.verify_ssl_src = rc.get("verify_ssl_src", True)
        self.verify_ssl_dst = rc.get("verify_ssl_dst", True)
        self.verbose = rc.get("verbose", False)
        self.clean = rc.get("clean", None)
        self.dry_run = rc.get("dry_run", False)
        self.ignore_missing = rc.get("ignore_missing", False)
        self.username = rc.get("username", os.environ.get("ANACONDA_MIRROR_USERNAME"))
        self.password = rc.get("password", os.environ.get("ANACONDA_MIRROR_PASSWORD"))
        self.token = rc.get("token", os.environ.get("ANACONDA_MIRROR_TOKEN"))
        self.dependencies = rc.get("dependencies", False)
        self.pruning = rc.get("pruning", False)
        self.auto_freeze = rc.get("auto_freeze", True)
        for var_name in (
            "channels",
            "platforms",
            "python_versions",
            "r_versions",
            "pkg_list",
            "license_exclusions",
            "constraints",
            "exclusions",
            "inclusions",
        ):
            setattr(self, var_name, _get_str_list(rc.get(var_name)))
        for var_name, value in rc.items():
            if not hasattr(self, var_name):
                setattr(self, var_name, value)
        self.check()

    def verify_ssl_mode(self, which):
        assert which in ("src", "dst")
        verify_ssl = self.verify_ssl
        verify_ssl_w = getattr(self, "verify_ssl_" + which)
        if (
            not verify_ssl
            or not verify_ssl_w
            or which == "src"
            and not context.ssl_verify
        ):
            return False
        if isinstance(verify_ssl_w, str):
            return verify_ssl_w
        if isinstance(verify_ssl, str):
            return verify_ssl
        if which == "src" and isinstance(context.ssl_verify, str):
            return context.ssl_verify
        return True

    def check(self, finalize=False):
        cvalue = self.channel_alias or conda_channel_alias()
        if cvalue.startswith("file://"):
            cvalue = cvalue[7:]
        if "://" not in cvalue:
            cvalue = "file://" + abspath(expanduser(cvalue))
        cvalue = cvalue.rstrip("/") + "/"
        self.channel_alias = cvalue

        if self.clean is not None and not finalize:
            raise ConfigError("--clean may only be specified on the command line")

        if self.token:
            if self.password:
                raise ConfigError("Must not supply both --token and --password")
            elif finalize:
                self.password = self.token
                self.token = None

        self.channels = _get_str_list(self.channels)
        if finalize:
            if not self.channels:
                raise ConfigError("At least one channel must be supplied")
            new_channels = []
            while self.channels:
                url = self.channels.pop(0)
                if url == "defaults":
                    self.channels[0:0] = conda_default_channels()
                    continue
                if url.startswith((".", "~", "/")):
                    url = "file://" + abspath(expanduser(url))
                elif "://" not in url:
                    url = cvalue + url
                if url.startswith(cvalue):
                    url = url[len(cvalue) :]
                elif url.startswith("file://"):
                    url = url[7:]
                new_channels.append(url)
            self.channels = new_channels

        if finalize and not self.dest_channel:
            raise ConfigError("Destination channel must be supplied")

        if self.platforms:
            all_platforms = known_subdirs()
            self.platforms = sorted(_get_str_list(self.platforms))
            for platform in self.platforms:
                if platform not in all_platforms:
                    raise ConfigError(
                        "Invalid platform: {}\nValid platforms: {}".format(
                            platform, ", ".join(all_platforms)
                        )
                    )
            if finalize:
                has_noarch = "noarch" in self.platforms or not self.strict_platforms
                self.platforms = sorted(set(self.platforms) - {"noarch"})
                if has_noarch:
                    self.platforms.append("noarch")

        ver_matcher = re.compile(r"\d+(.\d+)+$")
        for field in ("python_versions", "r_versions"):
            vlist = getattr(self, field) or []
            for ver in vlist:
                if not ver_matcher.match(ver):
                    raise ConfigError(
                        "Invalid version string in {}: {}".format(field, ver)
                    )
        if "3.1" in self.python_versions:
            vlist = (set(self.python_versions) - {"3.1"}) | {"3.10"}
            self.python_versions = sorted(vlist)

        for license_family in self.license_exclusions:
            if license_family not in standard_license_families:
                raise ConfigError(
                    "Invalid license family: {}\nKnown license families: {}".format(
                        license_family, standard_license_families
                    )
                )

        for field in ("pkg_list", "constraints", "exclusions", "inclusions"):
            mlist = getattr(self, field) or []
            for k, pkg in enumerate(mlist):
                try:
                    mlist[k] = MatchSpec(pkg)
                except Exception:
                    raise ConfigError(
                        "Invalid match string in {}: {}".format(field, pkg)
                    )

        if self.access_policy and self.access_policy not in all_access_policies:
            raise ConfigError(
                "Invalid access policy: {}\nValid access policies: {}".format(
                    self.access_policy, ", ".join(all_access_policies)
                )
            )

        if self.format_policy and self.format_policy not in all_format_policies:
            raise ConfigError(
                "Invalid format policy: {}\nValid format policies: {}".format(
                    self.format_policy, ", ".join(all_format_policies)
                )
            )

    def show(self):
        print("Configuration")
        print("-------------")
        for config_field in (
            "channel_alias",
            "channels",
            "platforms",
            "dest_site",
            "dest_channel",
            "format_policy",
            "ignore_missing",
            "max_attempts",
            "max_failures",
            "python_versions",
            "r_versions",
            "pkg_list",
            "dependencies",
            "pruning",
            "license_exclusions",
            "constraints",
            "exclusions",
            "inclusions",
            "clean",
        ):
            value = getattr(self, config_field)
            if isinstance(value, list):
                print("{}:{}".format(config_field, "" if value else " []"))
                for v in value:
                    print("  - {}".format(v))
            else:
                print("{}: {}".format(config_field, value))


def parse_argv():
    global config

    p = argparse.ArgumentParser(
        description="The official mirroring tool for Anaconda repositories",
        epilog="Versions: anaconda-mirror %s, conda %s" % (__version__, conda_version),
    )
    p.add_argument(
        "-f",
        "--file",
        dest="filename",
        action="store",
        help="Path to the configuration file. Note that the filename can "
        "also be supplied as a positional argument without -f/--file.",
    )
    p.add_argument(
        "--dest-channel",
        dest="dest_channel",
        action="store",
        default=None,
        help="The name of the destination channel.",
    )
    p.add_argument(
        "--channel",
        dest="channels",
        action="append",
        help="A source channel, given as either a short name or a full URL. "
        "Multiple channels may be supplied by repeated use of this option.",
    )
    p.add_argument(
        "--channel-alias",
        default=None,
        help="A URL to prepend to all short source channel names. If not "
        "supplied, the value configured within conda will be used.",
    )
    p.add_argument(
        "--dry-run",
        dest="dry_run",
        action="store_true",
        default=None,
        help="No upload, removal, or patch action should be taken. Instead, "
        "just show what steps would be taken during normal operation.",
    )
    p.add_argument(
        "-v",
        "--verbose",
        default=None,
        action="store_true",
        help="Provide more verbose output.",
    )
    p.add_argument(
        "-c",
        "--clean",
        default=None,
        action="store_true",
        help="Packages that no longer exist at the source will be removed "
        "from the destination. To ensure this is not accidentally invoked, "
        "it can only be included on the command line as, not in a config file. "
        "We recommend using the --dry-run option first to confirm what "
        "deletions would actually take place.",
    )
    p.add_argument(
        "--platform",
        dest="platforms",
        action="append",
        help="The name of a platform to mirror. Multiple platforms "
        "may be supplied by repeated use of this option. If none are "
        "supplied, all platforms known to conda will be included. The "
        "noarch platform will be included in any platform selection, "
        "unless --strict_platforms is also invoked.",
    )
    p.add_argument(
        "--strict-platforms",
        action="store_true",
        default=None,
        help="By default, the noarch platform is included in any mirror, "
        "since its packages are compatible with all platforms. If this flag "
        "is supplied, it will be included. So if you seek to truly mirror "
        "particular platform subdirectories, you may need this option.",
    )
    p.add_argument(
        "--format-policy",
        default=None,
        help="Determines what formats should be delivered to the destination. "
        "Values include: keep-both, prefer-conda, prefer-tarbz2, only-conda, "
        "only-tarbz2, transmute-conda, transmute-tarbz2. Not all destinations "
        "support the .conda package format, so the default is prefer-conda for "
        "destinations that can, and transmute-bz2 otherwise.",
    )
    p.add_argument(
        "--python-version",
        action="append",
        dest="python_versions",
        help="Filter the package set by Python version. Multiple versions "
        "can be supplied by repeated use of this option. By default, all "
        "python versions are included. Packages that do not depend on python "
        "are unaffected by this filter.",
    )
    p.add_argument(
        "--r-version",
        action="append",
        dest="r_versions",
        help="Filter the package set by R version. Multiple versions "
        "can be supplied by repeated use of this option. By default, all "
        "R versions are included. Packages that do not depend on R are "
        "unaffected by this filter.",
    )
    p.add_argument(
        "--include-only",
        "--pkg-list",
        action="append",
        dest="pkg_list",
        help="A package name or conda match pattern. Multiple items can be "
        "supplied by repeated use of this option. If supplied, the mirror "
        "will only include packages that match one or more of these patterns; "
        "and if --dependencies is selected, dependencies as well. This filter "
        "is applied prior to any exclusion/inclusion filtering.",
    )
    p.add_argument(
        "--dependencies",
        action=argparse.BooleanOptionalAction,
        default=None,
        help="The mirror will include not just the items matching "
        "the --include-only/--pkg-list values, but their dependencies as well. "
        "This only applies if --include-only/--pkg-list was used.",
    )
    p.add_argument(
        "--pruning",
        action=argparse.BooleanOptionalAction,
        default=None,
        help="Packages that are uninstallable due to missing dependencies "
        "will be removed from the mirror. If --include-only/--pkg-list is "
        "invoked, and one or more of those matches is pruned away "
        "completely, the mirror will not proceed.",
    )
    lstrs = ", ".join(standard_license_families)
    p.add_argument(
        "--license-exclude",
        action="append",
        dest="license_exclusions",
        help="Excludes packages with a license from the given license family. "
        "Multiple exclusions may be supplied by repeated use of this option. "
        "Possible values include: " + lstrs + ".",
    )
    p.add_argument(
        "--constraint",
        action="append",
        dest="constraints",
        help="Constrains packages according to the conda patch pattern given. "
        "Unlike standard conda match behavior, each constraint is applied only to "
        "packages that match by name. So for instance, python=3.9 will block "
        "python 3.10, but not plotly 3.10. Multiple constraints may be supplied by "
        "repeated use of this option.",
    )
    p.add_argument(
        "--exclude",
        action="append",
        dest="exclusions",
        help="Exclude packages that match the given conda match pattern from "
        "the mirror. Multiple exclusions may be supplied by repeated use "
        "of this option.",
    )
    p.add_argument(
        "--include",
        action="append",
        dest="inclusions",
        help="Include packages that match the supplied conda match expression, "
        "even though they would otherwise be excluded by a --filter, --exclude, "
        "or --license-exclude expression. This is used to make exceptions to a "
        "more blanket exclusion list. Packages filtered out by the --pkg-list "
        "or --include-only options are not affected. Multiple exceptions may be "
        "supplied by repeated use of this option.",
    )
    p.add_argument(
        "--ignore-missing",
        default=False,
        action=argparse.BooleanOptionalAction,
        help="If a package is listed in the destination index but the file "
        "is not present, do not re-retrieve it. This is useful for certain "
        "airgap scenarios to build incremental mirror bundles. "
        "Applies to local, s3, nexus, and jfrog targets only.",
    )
    p.add_argument(
        "--max-attempts",
        action="store",
        type=int,
        default=None,
        help="The maximum number of times to attempt a download or upload in "
        "the face of unexpected connection issues. (default: 5)",
    )
    p.add_argument(
        "--max-failures",
        action="store",
        type=int,
        default=None,
        help="The maximum number of packages that can fail the download/upload "
        "process before the mirror process stops. (default: 100)",
    )
    p.add_argument(
        "--verify-ssl",
        default=True,
        action=argparse.BooleanOptionalAction,
        help="Enable/disable SSL verification on both the source and the destination. "
        "On the command line it is a boolean flag, but in a config file you may set it "
        "to the string providing the path to a root CA trust file.",
    )
    p.add_argument(
        "--verify-ssl-src",
        default=None,
        action=argparse.BooleanOptionalAction,
        help="Enable/disable SSL verification on the source. "
        "On the command line it is a boolean flag, but in a config file you may set it "
        "to the string providing the path to a root CA trust file.",
    )
    p.add_argument(
        "--verify-ssl-dst",
        default=None,
        action=argparse.BooleanOptionalAction,
        help="Enable/disable SSL verification on the destination. "
        "On the command line it is a boolean flag, but in a config file you may set it "
        "to the string providing the path to a root CA trust file.",
    )
    p.add_argument(
        "--dest-site",
        dest="dest_site",
        action="store",
        default=None,
        help="The name of the destination site. If you have multiple "
        "destination repositories configured, this option allows you "
        "to select between them. The format of this site name depends "
        "upon the destination API.",
    )
    p.add_argument(
        "--username",
        default=None,
        help="The username used to authenticate to the destination repository. "
        "The precise interpretation depends upon the destination API. Can also "
        "be supplied in the environment variable ANACONDA_MIRROR_USERNAME, or "
        "using a standard ~/.netrc file.",
    )
    p.add_argument(
        "--password",
        "--token",
        default=None,
        help="The password, API token, or bearer token used to authenticate to "
        "the destination repository. The precise interpretation depends upon the "
        "destination API. Can also be supplied in the environment variable "
        "ANACONDA_MIRROR_PASSWORD or ANACONDA_MIRROR_TOKEN, or using a "
        "standard ~/.netrc file.",
    )
    p.add_argument(
        "--auto-freeze",
        action=argparse.BooleanOptionalAction,
        default=None,
        help="(Anaconda Server 6 only.) By default, anaconda-mirror freezes the "
        "channel index during the mirroring process, and un-freezes it upon "
        "completion, to improve performance. To disable this behavior, use "
        "the --no-auto-freeze option.",
    )
    p.add_argument(
        "--config",
        action="store_true",
        help="show the configuration information and exit.",
    )
    p.add_argument("args", nargs="?")

    opts = p.parse_args()
    args = []
    if opts.args:
        args.append(opts.args)
    if opts.filename:
        args.append(opts.filename)
    if args and any(args[0] != a for a in args[1:]):
        p.error("%d filenames supplied: %s" % (len(args), ",".join(args)))

    if args:
        try:
            config.__init__(path=args[0])
        except ConfigError as exc:
            p.error(str(exc))
    opts = vars(opts)
    for attr, value in opts.items():
        if value is not None and hasattr(config, attr):
            if isinstance(value, list):
                getattr(config, attr).extend(value)
            else:
                setattr(config, attr, value)
    try:
        config.check(finalize=True)
    except ConfigError as exc:
        p.error(str(exc))

    do_exit = False
    if opts["config"]:
        if do_exit:
            print("")
        do_exit = True
        config.show()
    if do_exit:
        print("")
        sys.exit(0)


config = Config()
